ace.require("ace/ext/language_tools");

var editor;
var editor2;

const lsPrefix = "tmy_";

const version = "V0.1 p";
const datum = "03.05.2022";

var oLastUrl = null;

var remote = 0;

var fileHandle = null;

var CONTENT_SEPARATOR = "###|*|###";


document.write = function (txt) {
    addConsoleMsg(txt);
}

document.writeln = function (txt) {
    addConsoleMsg(txt + "<br>");
}

console.log = function (txt) {
    addConsoleMsg(txt + "<br>");
}

function getActDate() {
    let ret = '';
    let d = new Date();
    const t = d.getDate();
    const m = d.getMonth();
    const y = d.getFullYear();
    ret = (t < 10 ? '0' + t : t) + "." + (m < 10 ? '0' + m : m) + "." + y;
    return ret;
}


function getActTime() {
    let ret = '';
    let d = new Date();
    const h = d.getHours();
    const m = d.getMinutes();
    const s = d.getSeconds();
    ret = (h < 10 ? '0' + h : h) + ":" + (m < 10 ? '0' + m : m) + ":" + (s < 10 ? '0' + s : s);
    return ret;
}


$(function () {

    var urlParams = new URLSearchParams(window.location.search);
    if (urlParams.has('rm') && urlParams.get('rm') == 1) {
        remote = true;
    }

    $('.version').empty().append(version);
    $('.datum').empty().append(datum);

    $('header, nav, main, footer, aside, .splitter-horizontal').hide();

    editor = ace.edit("jsinput");
    editor.setTheme("ace/theme/chrome");
    editor.session.setMode("ace/mode/javascript");

    editor.session.on('change', function (delta) {
        update();
    });

    editor.on('focus', function () {
        lsg.clearSelectionMarkers();
    });

    editor.setOptions({
        enableBasicAutocompletion: true,
        enableSnippets: true,
        enableLiveAutocompletion: false,
        fontSize: "1em"
    });


    editor2 = ace.edit("guiinput");
    editor2.setTheme("ace/theme/chrome");
    editor2.session.setMode("ace/mode/html");

    editor2.session.on('change', function (delta) {
        updateGUI();
    });

    editor2.on('focus', function () {
        // console.log("editor2: focus")
    });

    editor2.setOptions({
        enableBasicAutocompletion: true,
        enableSnippets: true,
        enableLiveAutocompletion: false,
        fontSize: "1em"
    });



    window.setTimeout(function () {
        $('.stat').on('click', function () {
            getStat();
        });
        getStat();
    }, 2000);


    $(".panel-left").resizable({
        handleSelector: ".splitter",
        resizeHeight: false,
        onDragEnd: function (e, $el, opt) {
            editor.resize();
            editor.renderer.updateFull();
            editor2.resize();
            editor2.renderer.updateFull();
        }
    });

    $(".panel-top").resizable({
        handleSelector: ".splitter-horizontal",
        resizeWidth: false,
        onDragEnd: function (e, $el, opt) {
            editor.resize();
            editor.renderer.updateFull();
            editor2.resize();
            editor2.renderer.updateFull();
        }
    });

    $('#btnDebugclose').on('click', function () {
        $('.highlightblock').removeClass('highlightblock');
        $('#debugdiv').hide();
    });

    $('#btnInOutclose').on('click', function () {
        $('#mbtnViewInOut').trigger('click');
    });


    initMenu();
    document.getElementById("defaultOpen").click();

    document.getElementById("infoBlockDefaultOpen").click();

    $('.closeTask').on('click', function () {
        $(this).closest('button')[0].style.display = "none";
        clearTask();
        $('#infoBlockDefaultOpen').trigger('click');
    });


    lsg.go(document.getElementById('struktogrammoutput'));

    let ts = loadCode(0);
    if (ts != null && ts.length > 0) {
        addInfoMsg("Programm geladen (Version vom " + ts + ")")
    }


    $('#btnDownload').on('click', function () {

    });

    window.setTimeout(function () {
        $('.stage').fadeOut(0, function () {
            $('body')[0].style.background = "none";
            if (remote == 1) {
                if (urlParams.has('code')) {
                    var output = the.beautifier.js(decodeURIComponent(escape(window.atob(urlParams.get('code')))), opts);
                    editor.setValue(output);
                }
            }
            lsg.refresh(editor, editor2);
            $('header, nav, main, footer, aside, .splitter-horizontal').fadeIn(0);
        });
    }, (1 - remote) * 0);

    //Offline
    $('.panel-right').hide();
    $('.panel-left')[0].style.width = "100%";


});




function initMenu() {
    //Info
    document.getElementById('infotext').innerHTML = infotext;

    var modal = document.getElementById("myModal");
    var btn = document.getElementById("mbtnInfo");
    btn.onclick = function () {
        modal.style.display = "block";
    }
    $('.close').on('click', function () {
        modal.style.display = "none";
    });
    window.onclick = function (event) {
        if (event.target == modal) {
            modal.style.display = "none";
        }
    }

    //shortcuts
    document.getElementById('shortcuts').innerHTML = shortcuts;
    var modalsc = document.getElementById("myModalShortcuts");
    var btnsc = document.getElementById("mbtnShortcut");
    btnsc.onclick = function () {
        modalsc.style.display = "block";
    }
    $('.close').on('click', function () {
        modalsc.style.display = "none";
    });
    window.onclick = function (event) {
        if (event.target == modalsc) {
            mmodalscodal.style.display = "none";
        }
    }


    //Tasks
    // Get the modal
    var mdtModal = document.getElementById("mdTasks");

    // Get the <span> element that closes the modal
    var mdtSpan = document.getElementsByClassName("mdTasksClose")[0];

    // When the user clicks on <span> (x), close the modal
    mdtSpan.onclick = function () {
        mdtModal.style.display = "none";
    }

    // When the user clicks anywhere outside of the modal, close it
    window.onclick = function (event) {
        if (event.target == mdtModal) {
            mdtModal.style.display = "none";
        }
    }

    if (!window.showOpenFilePicker) {
        $('.nativeFS').hide();
        addInfoMsg0("<div style='color:blue;margin: 10px 0;'><u><strong>Tipp</strong></u>: Aktivieren Sie den Dateizugriff, um Ihre Ergebnisse im Dateisystem zu speichern. Rufen Sie dazu in einem neuen Tab die Adresse 'chrome://flags/#native-file-system-api' auf und setzen Sie den Wert für den Eintrag 'Native File System API' auf 'Enabled'.</div>");
        addInfoMsg("Dateizugriff nicht aktiviert (--&gt; chrome://flags/#native-file-system-api)");
    }


    //Others
    $('#mbtnViewCode').on('click', function () {
        /*
        if ($(this).is(':checked')) {
            $('.panel-left').show();
            $('.panel-right')[0].style.width = "100%";
        } else {
            $('.panel-left').hide();
        }

        if ($('#mbtnViewCode').is(':checked') && $('#mbtnViewStruktogramm').is(':checked')) {
            $('.panel-left')[0].style.width = "50%";
            $('.panel-right')[0].style.width = "50%";
        }
        */
    });

    $('#mbtnViewStruktogramm').on('click', function () {
        /*
        if ($(this).is(':checked')) {
            $('.panel-right').show();
        } else {
            $('.panel-right').hide();
            $('.panel-left')[0].style.width = "100%";
        }

        if ($('#mbtnViewCode').is(':checked') && $('#mbtnViewStruktogramm').is(':checked')) {
            $('.panel-left')[0].style.width = "50%";
            $('.panel-right')[0].style.width = "50%";
        }
        */
    });

    $('#mbtnDemoCode').on('click', function () {
        setEditorValue(democode);
        editor.resize();
        editor.renderer.updateFull();
    });

    $('#mbtnDemoCode2').on('click', function () {
        setEditorValue(democode2);
        editor.resize();
        editor.renderer.updateFull();
    });



    $('#mbtnBeautify').on('click', function () {
        var output = the.beautifier.js(editor.getValue(), opts);
        editor.setValue(output);
        var output2 = the.beautifier.html(editor2.getValue(), opts);
        editor2.setValue(output2);
        updateGUI();
    });

    $('#mbtnAutoscroll').on('click', function () {
        lsg.autoscroll = $(this)[0].checked;
    });

    $('#mbtnAutogui').on('click', function () {
        lsg.autogui = $(this)[0].checked;
        lsg.refresh(editor, editor2);
    });

    $('#mbtnExtendedStruktogramm').on('click', function () {
        lsg.setSubtextvisibility($(this)[0].checked);
    });


    $('#mbtnViewInOut').on('click', function () {
        if ($(this)[0].checked) {
			console.log($('#inoutdiv')[0].style.top.substr(0, $('#inoutdiv')[0].style.top.length-2));
			if($('#inoutdiv')[0].style.top.substr(0, $('#inoutdiv')[0].style.top.length-2) < 0) {
				$('#inoutdiv')[0].style.top = "100px";
			}
            $('#inoutdiv').show();
        } else {
            $('#inoutdiv').hide();
        }
    });

    $('#mbtnclearcode').on('click', function () {
        fileHandle = null;
        clearEditor();
    });

    $('#mbtnExec').on('click', function () {
        clearConsole();
        clearError();
        saveCode(0);
        addInfoMsg("Programm automatisch gespeichert");
        $('#thescript').empty();
        $('#thescript').append("<script type='text/javascript'>" + $('<textarea />').html(lsg.getFunctions()).text() + "</script>");

        lsg.execute();
    });

    $('#mbtnReload').on('click', function () {
        $('header, nav, main, footer, aside, .splitter-horizontal').fadeOut(0, function () {
            location.reload();
        });
    });

    $('#mbtnSave').on('click', function () {
        saveCode(0);
		
		if(fileHandle == null) {
			addInfoMsg("Programm gespeichert");
		} else {
			saveFile();
		}
		
    });


    $('#mbtnLoad').on('click', function () {
        let ts = loadCode(0);
        if (ts != null && ts.length > 0) {
            addInfoMsg("Programm geladen (Version vom " + ts + ")")
        }
    });


    $('#mbtnSaveLocal').on('click', async (e) => {
		fileHandle = await getNewFileHandle();
        saveFile();
    });

    $('#mbtnOpenLocal').on('click', async (e) => {
        if (window.showOpenFilePicker) {
			
			if($('#mbtnAutogui')[0].checked) {
				$('#mbtnAutogui').trigger('click');
				addInfoMsg("Autom. GUI-Generierung deaktiviert");
			}
			
		[fileHandle] = await window.showOpenFilePicker();
            const file = await fileHandle.getFile();
            const contents = await file.text();
			console.log(contents);
            var tokens = contents.split(CONTENT_SEPARATOR);
            if (tokens.length > 0) {
                clearEditor();
                editor.setValue(tokens[0]);
            }
            if (tokens.length > 1) {
                editor2.setValue(tokens[1]);
            }
            lsg.refresh(editor, editor2);
            addInfoMsg("Programm " + fileHandle.name + " geladen");
            showSnackbar("Programm geladen");
			document.getElementsByTagName('title')[0].innerText = "Thommy - " + fileHandle.name;
        } else {
            addInfoMsg('Sie müssen den Dateizugriff aktivieren. Rufen Sie dazu in Google Chrome die folgende Adresse auf: chrome://flags/#native-file-system-api (Copy&Paste).');
        }
    });

    $('#mbtnPDF').on('click', function () {
        saveCode(0);

        showSnackbar("PDF-Export wird erstellt ...");

        addInfoMsg("Programm gespeichert");
        addInfoMsg("PDF-Export erstellt");
        var pdf = new jsPDF({
            orientation: "portrait",
            unit: "mm",
            format: "a4"
        });

        pdf.setFont('arial', 'normal');
        pdf.setFontSize(10);
        pdf.text("Ausdruck vom " + getActDate() + " " + getActTime(), 10, 10);

        pdf.setFont('arial', 'normal');
        pdf.setFontSize(8);
        pdf.text("Thommy " + version, 180, 290);

        if ($('#struktogrammoutput:visible').length > 0) {
            html2canvas(document.getElementById('struktogrammoutput')).then(function (canvas) {
                let img = new Image();
                img.src = canvas.toDataURL("image/png");
                img.addEventListener("load", function () {
                    //pdf.addImage(img, 'png', 20, 20, canvas.width, canvas.height);

                    if ($('.panel-left:visible').length > 0) {
                        pdf.addImage(canvas, 'png', 110, 20, 90, 90 * canvas.height / canvas.width);

                        pdf.setFont('monospace', 'normal');
                        var lines = pdf.setFontSize(8).splitTextToSize(editor.getValue(), 100);
                        pdf.text(10, 20 + 8 / 100, lines);
                    } else {
                        pdf.addImage(canvas, 'png', 20, 20, 170, 170 * canvas.height / canvas.width);
                    }

                    pdf.save("thommy-export.pdf");
                });

            });
        } else {
            if ($('.panel-left:visible').length > 0) {
                pdf.setFont('monospace', 'normal');
                var lines = pdf.setFontSize(8).splitTextToSize(editor.getValue(), 280);
                pdf.text(10, 20 + 8 / 280, lines);
            }

            pdf.save("thommy-export.pdf");
        }

    });

    $('#mbtnPub').on('click', function () {
 var sText;


	sText = "";
	sText += "<!--\n";
	sText += "############################################################\n";
	sText += "###   Generiert von Thommy " + version + " am " + getActDate() + " " + getActTime() + "   ###\n";
	sText += "############################################################\n";
	sText += "-->\n\n\n";
	sText += "<html>\n";
	sText += "  <head>\n";
	sText += "    <title>Thommy-HTML-Vorlage</title>\n";
	sText += "    <meta charset='utf-8'>\n\n";
	sText += "    <meta name='viewport' content='width=device-width, initial-scale=1.0, user-scalable=no'></meta>";
	sText += "  </head>\n";
	sText += "  <body id='inputoutput'>\n";
	sText += the.beautifier.html(editor2.getValue(), opts);
	sText += "    \n\n\n";
	sText += "    <script id='thommy'>\n";
	sText += "    function init() {\n\n";
	sText += "    document.querySelectorAll('.button').forEach(function(b){b.disabled=false;});\n\n\n";
	sText += "    }\n\n\n";
	sText += "    // ##########################################################\n";
	sText += "    // ##########   HIER BEGINNT IHR JAVASCRIPT-CODE   ##########\n";
	sText += "    // ##########################################################\n\n";
	sText += the.beautifier.js(editor.getValue(), opts);
	sText += "\n\n        // #########################################################\n";
	sText += "        // ##########   HIER ENDET IHR JAVASCRIPT-CODE   ##########n\n";
	sText += "        // #########################################################\n\n";
	sText += "    \n\n";
	// sText +=    lsg.getFunctions();
	sText += "  </script>\n";
	sText += "  </body>\n";
	sText += "</html>";

	sText = the.beautifier.html(sText);
	
        var fn = "";
        if(localStorage.getItem(lsPrefix + "pubname") != null) {
            fn = localStorage.getItem(lsPrefix + "pubname");
        }

        $.post(
            '/pub/upload.php',
            {
                data: sText,
                fn: fn
            },
            function (data) {
                if(data.startsWith("fn:")) {
                    localStorage.setItem(lsPrefix + "pubname", data.substring(3));
                    var d = (new Date()).getDate();
                    if(d < 10){
                        d = '0'+d;
                    }
                    showSnackbar("Seite publiziert");
                    addInfoMsg("Seite publiziert: <a href='http://" + window.location.hostname + "/pub/" + d + "/" + localStorage.getItem(lsPrefix + "pubname") + ".html' target='_blank'>http://" + window.location.hostname + "/pub/" + d + "/" + localStorage.getItem(lsPrefix + "pubname") + ".html</a>");
                } else {
                    localStorage.removeItem(lsPrefix + "pubname");
                    showSnackbar("Seite nicht publiziert");
                    addInfoMsg("Seite konnte nicht erfolgreich publiziert werden");
                }
            },
            "html"
        );


    });


    $('#mbtnHTML').on('click', function () {
        var sText, oBlob, elLink;

        if (oLastUrl == null) {
            window.URL.revokeObjectURL(oLastUrl);
            oLastUrl = null;
        }

        sText = "";
        sText += "<!--\n";
        sText += "############################################################\n";
        sText += "###   Generiert von Thommy " + version + " am " + getActDate() + " " + getActTime() + "   ###\n";
        sText += "############################################################\n";
        sText += "-->\n\n\n";
        sText += "<html>\n";
        sText += "  <head>\n";
        sText += "    <title>Thommy-HTML-Vorlage</title>\n";
        sText += "    <meta charset='utf-8'>\n\n";
        sText += "  <!-- ##########   Folgendes style-Element dient zur Formatierung der Seite mittels CSS   ########## -->\n";
        sText += "    <style>";
        sText += "      body {";
        sText += "          margin: 10px; padding: 0px;font-family: Arial, Helvetica, sans-serif;";
        sText += "      }";
        sText += "      header {";
        sText += "          border-bottom: 1px solid black;margin-bottom: 30px;";
        sText += "      }";
        sText += "      fieldset {";
        sText += "         display: inline-block;color: lightgray;border: 1px solid lightgray;";
        sText += "      }";
        sText += "      #inout td {";
        sText += "      	padding: 4px;";
        sText += "      }";
        sText += "      #exec {";
        sText += "      margin: 32px 0;";
        sText += "      }";
        sText += "      </style>\n\n";
        sText += "  </head>\n";
        sText += "  <body>\n";
        sText += "  <!-- ##########   SEITENKOPF   ########## -->\n\n";
        sText += "    <header>";
        sText += "    <h1>HTML-Vorlage</h1>";
        sText += "    <h6>Generiert von Thommy " + version + " am " + getActDate() + " " + getActTime() + "</h6>";
        sText += "    </header>";
        sText += "  \n\n\n\n\n";
        sText += "  <!-- ##########   BEGINN Ein-/Ausgabefelder   ########## -->\n\n";
        sText += "  <fieldset><legend>Ein-/Ausgabe</legend>\n";
        // sText += "  <table id='inout'>\n";
        sText += $('#inputoutput').html();
        // sText += "  </table>\n";
        sText += "  </fieldset>\n\n";
        sText += "  <!-- ##########   ENDE Ein-/Ausgabefelder   ########## -->";
        sText += "  \n\n\n\n\n";
        sText += "    <script id='thommy'>\n";
        sText += "    function init() {\n\n";
        sText += "    document.querySelectorAll('.button').forEach(function(b){b.disabled=false;});\n\n\n";
        sText += "    // ##########################################################\n";
        sText += "    // ##########   HIER BEGINNT IHR JAVASCRIPT-CODE   ##########\n";
        sText += "    // ##########################################################\n\n";
        sText += the.beautifier.js(editor.getValue(), opts);
        sText += "\n\n        // #########################################################\n";
        sText += "        // ##########   HIER ENDET IHR JAVASCRIPT-CODE   ##########n\n";
        sText += "        // #########################################################\n\n";
        sText += "    }\n";
        sText += "    \n\n";
        // sText +=    lsg.getFunctions();
        sText += "  </script>\n";
        sText += "  \n\n\n\n\n";
        sText += "  <!-- ##########   STARTKNOPF   ########## -->\n\n";
        sText += "  <div><button id='exec' onclick='init()'>Programm starten</button></div>\n\n";
        sText += "  </body>\n";
        sText += "</html>";

        sText = the.beautifier.html(sText);

        oBlob = new Blob([sText], { type: "text/html" });

        if (window.navigator.msSaveBlob) {
            window.navigator.msSaveBlob(oBlob, "Vorlage.html");
        } else {
            elLink = document.getElementById("downloadLink");
            oLastUrl = window.URL.createObjectURL(oBlob);
            elLink.href = oLastUrl;
            elLink.click();
        }

        showSnackbar("HTML-Vorlage erstellt");
        addInfoMsg("HTML-Vorlage erstellt");
    });

    $('#mbtnTDemo').on('click', function () {
        task = tdemoV1;
        showTask();
    });

    $('#mbtnASBWBW').on('click', function () {
        task = bgbwV1;
        showTask();
    });

    document.onkeydown = function (e) {
        //console.log(e);
        if (e.ctrlKey && !e.shiftKey && e.which == 122) {
            $('#mbtnExec').trigger('click');
        } else if (e.altKey && !e.shiftKey && e.which == 122) {
            $('#run').trigger('click');
        } else if (e.ctrlKey && e.which == 66) {
            $('#mbtnBeautify').trigger('click');
        } else if (e.ctrlKey && e.which == 73) {
            $('#mbtnInfo').trigger('click');
        } else if (e.ctrlKey && e.which == 13) {
            $('#mbtnPub').trigger('click');
        } else if (e.altKey && e.which == 84) {
            $('#mbtnShortcut').trigger('click');
        } else if (e.ctrlKey && e.which == 83) {
            $('#mbtnSave').trigger('click');
            stopEventPropagation(e);
        } else if (e.altKey && e.which == 83) {
            $('#mbtnSaveLocal').trigger('click');
            stopEventPropagation(e);
        } else if (e.ctrlKey && e.which == 79) {
            $('#mbtnLoad').trigger('click');
            stopEventPropagation(e);
        } else if (e.altKey && e.which == 79) {
            $('#mbtnOpenLocal').trigger('click');
            stopEventPropagation(e);
        } else if (e.altKey && e.which == 80) {
            $('#mbtnPDF').trigger('click');
            stopEventPropagation(e);
        } else if (e.altKey && e.which == 72) {
            $('#mbtnHTML').trigger('click');
            stopEventPropagation(e);
        } else if (e.ctrlKey && e.which == 69) {
            $('#mbtnViewInOut').trigger('click');
            stopEventPropagation(e);
        }
    };



    dragElement(document.getElementById("debugdiv"));
    dragElement(document.getElementById("inoutdiv"));

    function dragElement(elmnt) {
        var pos1 = 0, pos2 = 0, pos3 = 0, pos4 = 0;
        if (document.getElementById(elmnt.id + "header")) {
            // if present, the header is where you move the DIV from:
            document.getElementById(elmnt.id + "header").onmousedown = dragMouseDown;
        } else {
            // otherwise, move the DIV from anywhere inside the DIV: 
            elmnt.onmousedown = dragMouseDown;
        }

        function dragMouseDown(e) {
            e = e || window.event;
            e.preventDefault();
            // get the mouse cursor position at startup:
            pos3 = e.clientX;
            pos4 = e.clientY;
            document.onmouseup = closeDragElement;
            // call a function whenever the cursor moves:
            document.onmousemove = elementDrag;
        }

        function elementDrag(e) {
            e = e || window.event;
            e.preventDefault();
            // calculate the new cursor position:
            pos1 = pos3 - e.clientX;
            pos2 = pos4 - e.clientY;
            pos3 = e.clientX;
            pos4 = e.clientY;
            // set the element's new position:
            elmnt.style.top = (elmnt.offsetTop - pos2) + "px";
            elmnt.style.left = (elmnt.offsetLeft - pos1) + "px";
        }

        function closeDragElement() {
            // stop moving when mouse button is released:
            document.onmouseup = null;
            document.onmousemove = null;
        }
    }
}

	
async function saveFile() {
	if(fileHandle  == null) {
		fileHandle = await showSaveFilePicker();
	}
	var content = editor.getValue() + CONTENT_SEPARATOR + editor2.getValue();
	await writeFile(fileHandle, content);
	addInfoMsg("Programm " + fileHandle.name + " gespeichert");
	showSnackbar("Programm gespeichert");
	document.getElementsByTagName('title')[0].innerText = "Thommy - " + fileHandle.name;
}


function getStat() {
    /*
    if (navigator.onLine) {
        $('.stat').load('/thommy/s/getlogo.php');
    }
    */
};

async function getNewFileHandle() {
    const opts = {
        type: 'save-file',
        accepts: [{
            description: 'Javascript-Datei',
            extensions: ['tmy'],
            mimeTypes: ['text/text'],
        }],
    };
    const handle = await window.showSaveFilePicker(opts);
    return handle;
}

async function writeFile(fileHandle, contents) {
    const writable  = await fileHandle.createWritable();
    await writable.truncate(0);
    await writable.write(contents);
    await writable.close();
}

function scrollSmoothToBottom(id) {
    var div = document.getElementById(id);
    $('#' + id).animate({
        scrollTop: div.scrollHeight - div.clientHeight
    }, 100);
}

function showSnackbar(msg) {
    $('#snackbar').text(msg);
    var x = document.getElementById("snackbar");
    x.className = "show";
    setTimeout(function () {
        x.className = x.className.replace("show", "");
        $('#snackbar').innerHTML = "";
    }, 3000);
}

function clearEditor() {
    document.getElementsByTagName('title')[0].innerText = "Thommy - " + version;
    editor.setValue('');
    editor.resize();
    editor.renderer.updateFull();
    editor2.setValue('');
    editor2.resize();
    editor2.renderer.updateFull();
}

function addError(txt) {
    $('#error').append(txt);
    scrollSmoothToBottom('infoblock');
    if (txt != null && txt.length > 0) {
        $('#infoblocktablinkHeaderError').text("Fehler !!!");
    }
}

function clearError() {
    $('#error').empty();
    $('#infoblocktablinkHeaderError').text("Fehler");
}

function setConsoleMsg(txt) {
    $('#Console').empty();
    addConsoleMsg(txt);
}

function addConsoleMsg(txt) {
    $('#Console').append(txt);
    scrollSmoothToBottom('infoblock');
}

function clearConsole() {
    $('#Console').empty();
}

function addInfoMsg0(txt) {
    $('#Info').append(txt + "<br>");
    scrollSmoothToBottom('infoblock');
}

function addInfoMsg(txt) {
    $('#Info').append(getActTime() + ": " + txt + "<br>");
    scrollSmoothToBottom('infoblock');
}

function clearInfo() {
    $('#Info').empty();
}

function setTaskMsg(txt) {
    $('#Task').empty().append(txt);
    scrollSmoothToBottom('infoblock');
}

function clearTask() {
    $('#Task').empty();
}

function updateGUI() {
    document.getElementById('inputoutput').innerHTML = editor2.getValue();
    $('#inputoutput button, #inputoutput input[type=button], #inputoutput input[type=submit]').addClass('button');
    $('#inputoutput button, #inputoutput input[type=button], #inputoutput input[type=submit]').attr('disabled', 'true');
}

function loadCode(id) {
    let ts = "";
    if (id == null || id == 0 || id.length <= 0) {
        if (localStorage.getItem(lsPrefix + "code") != null) {
            editor.setValue(localStorage.getItem(lsPrefix + "code"));
        }
        if (localStorage.getItem(lsPrefix + "codegui") != null) {
            editor2.setValue(localStorage.getItem(lsPrefix + "codegui"));
        }
        if (localStorage.getItem(lsPrefix + "autogui") != null) {
            $('#mbtnAutogui')[0].checked = localStorage.getItem(lsPrefix + "autogui");
            $('#mbtnAutogui').trigger('click');
        }
        if (localStorage.getItem(lsPrefix + "codets") != null) {
            ts = localStorage.getItem(lsPrefix + "codets");
        }

        lsg.refresh(editor, editor2);
    }
    return ts;
}

function saveCode(id) {
    if (id == null || id == 0 || id.length <= 0) {
        localStorage.setItem(lsPrefix + "code", editor.getValue());
        localStorage.setItem(lsPrefix + "codegui", editor2.getValue());
        localStorage.setItem(lsPrefix + "autogui", $('#mbtnAutogui')[0].checked);
        localStorage.setItem(lsPrefix + "codets", getActDate() + " " + getActTime());
        showSnackbar("Programmcode gespeichert");
    }
}


function setEditorValue(code) {
    if (editor.getValue().trim().length > 0) {
        if (confirm("Wollen Sie den bestehenden Code ersetzen?")) {
            editor.setValue(code);
        }
    } else {
        editor.setValue(code);
    }
    lsg.refresh(editor);
}


function setEditor2Value(code) {
    document.getElementById('inputoutput').innerHTML = code;
    editor2.setValue(the.beautifier.html($('#inputoutput')[0].innerHTML));
}


function getSiteCode() {
   var sText;


	sText = "";
	sText += "<!--\n";
	sText += "############################################################\n";
	sText += "###   Generiert von Thommy " + version + " am " + getActDate() + " " + getActTime() + "   ###\n";
	sText += "############################################################\n";
	sText += "-->\n\n\n";
	sText += "<html>\n";
	sText += "  <head>\n";
	sText += "    <title>Thommy-HTML-Vorlage</title>\n";
	sText += "    <meta charset='utf-8'>\n\n";
	sText += "    <meta name='viewport' content='width=device-width, initial-scale=1.0, user-scalable=no'></meta>";
	sText += "  </head>\n";
	sText += "  <body id='inputoutput'>\n";
	sText += the.beautifier.html(editor2.getValue(), opts);
	sText += "    \n\n\n";
	sText += "    <script id='thommy'>\n";
	sText += "    function init() {\n\n";
	sText += "    document.querySelectorAll('.button').forEach(function(b){b.disabled=false;});\n\n\n";
	sText += "    }\n\n\n";
	sText += "    // ##########################################################\n";
	sText += "    // ##########   HIER BEGINNT IHR JAVASCRIPT-CODE   ##########\n";
	sText += "    // ##########################################################\n\n";
	sText += the.beautifier.js(editor.getValue(), opts);
	sText += "\n\n        // #########################################################\n";
	sText += "        // ##########   HIER ENDET IHR JAVASCRIPT-CODE   ##########n\n";
	sText += "        // #########################################################\n\n";
	sText += "    \n\n";
	// sText +=    lsg.getFunctions();
	sText += "  </script>\n";
	sText += "  </body>\n";
	sText += "</html>";

	sText = the.beautifier.html(sText);
	
	return sText;
}	


function stopEventPropagation(e) {
    e.preventDefault();
    e.stopPropagation();
    window.event.returnValue = false;
}


function showTask() {
    let html = '';
    html += `<h3 class='tasksTitle'>${task.title}</h3>`;
    html += `<div class='tasksId'>ID: ${task.id}</div>`;
    html += `<div class='tasksDescription'>${task.description}</div>`;

    $('#mdTasksContentDetail').empty().append(html);

    if (task.tasks != null && task.tasks.length > 0) {
        for (let i = 0; i < task.tasks.length; i++) {
            const tableElement = document.createElement('table');
            tableElement.setAttribute('data-taskid', `${task.id}_${task.tasks[i].id}`);
            tableElement.setAttribute('class', 'taskTable');
            $('#mdTasksContentDetail').append(tableElement);

            $(`table[data-taskid=${task.id}_${task.tasks[i].id}]`).data('task', task.tasks[i]);

            html = `<table class='taskTable' data-taskid='${task.id}_${task.tasks[i].id}'>`;
            html += `<tr>`;
            html += `  <td colspan='3' class='taskTitle'>${task.tasks[i].pos}. ${task.tasks[i].title} (${task.tasks[i].lfs})</td>`;
            html += "</tr>";
            html += "<tr>";
            html += `  <td colspan='3' class='taskHint'><i class="fa fa-exclamation-triangle"></i> ${task.tasks[i].hint}</td>`;
            html += "</tr>";
            html += "<tr>";
            html += `  <td colspan='2' class='taskTask'>${task.tasks[i].task}</td>`;
            if (task.tasks[i].material != null && task.tasks[i].material.length > 0) {
                html += `<td><i class="fa fa-info-circle"></i>`;
                for (let m = 0; m < task.tasks[i].material.length; m++) {
                    html += `<div class='tasklink'><a href='${task.tasks[i].material[m].url}' target='_extern'>${task.tasks[i].material[m].title}</a></div>`;
                }
                html += `</td>`;
            } else {
                html += `<td>&nbsp;</td>`;
            }
            html += "</tr>";
            html += "<tr>";
            html += '  <td colspan="3" class="taskControlButtons">';
            html += '  <button class="setActiveTask">Aufgabe bearbeiten</button>';
            html += '  </td>';
            html += "</tr>";

            tableElement.innerHTML = html;
        }
    }


    $('.setActiveTask').on('click', function () {
        mdTasks.style.display = "none";
        setActiveTask($(this).closest('table').data('task'));
    })

    var mdtModal = document.getElementById("mdTasks");
    mdtModal.style.display = "block";
}

function setActiveTask(subtask) {
    if (subtask.template) {
        setEditorValue(subtask.template);
    }
    if (subtask.gui) {
        setEditor2Value(subtask.gui);
        $('#mbtnAutogui')[0].checked = false;
        if (!$('#mbtnViewInOut')[0].checked) {
            $('#mbtnViewInOut').trigger('click');
        }
    }

    if (subtask.guiwidth) {
        document.getElementById('inoutdiv').style.width = subtask.guiwidth;
    }
    if (subtask.guiheight) {
        document.getElementById('inoutdiv').style.height = subtask.guiheight;
    }

    var html = "";
    html += "<div class='taskwrapper'>";
    html += "  <div class='left'>";
    html += `    <h1>${subtask.pos}. ${subtask.title}  (${subtask.lfs}) - [${task.id}]</h1>`;
    html += `    <div class='task'>${subtask.task}</div>`;
    html += "  </div>";
    html += "  <div class='right'>";
    html += "    <button>Kurs anzeigen</button><br>";
    html += "    <button disabled>Lösung prüfen</button><br>";
    html += "    <div style='margin-top: 16px;'>Status: unbekannt</div>";
    html += "  </div>";
    html += "</div>";
    setTaskMsg(html);

    $('#Task button:first-child').on('click', function () {
        showTask();
    });

    document.getElementById('infoblocktablinkHeaderTitleTask').innerText = `Aktuelle Aufgabe (${subtask.pos})`;
    document.getElementById('infoblocktablinkHeaderTask').style.display = "inline-block";
    document.getElementById('infoblocktablinkHeaderTask').click();
}


function openInfoBlockTab(evt, name) {
    // Declare all variables
    var i, tabcontent, tablinks;

    // Get all elements with class="tabcontent" and hide them
    tabcontent = document.getElementsByClassName("infoblocktabcontent");
    for (i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = "none";
    }

    // Get all elements with class="tablinks" and remove the class "active"
    tablinks = document.getElementsByClassName("tablinks");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].className = tablinks[i].className.replace(" active", "");
    }

    // Show the current tab, and add an "active" class to the button that opened the tab
    document.getElementById(name).style.display = "block";
    evt.currentTarget.className += " active";
}

function openInfoBlockPage(pageName, elem) {
    // Hide all elements with class="tabcontent" by default */
    var i, tabcontent, tablinks;
    tabcontent = document.getElementsByClassName("infoblocktabcontent");
    for (i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = "none";
    }

    // Remove the background color of all tablinks/buttons
    tablinks = document.getElementsByClassName("infoblocktablink");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].style.backgroundColor = "";
    }
    elem.style.backgroundColor = '#2196F3';


    // Show the specific tab content
    document.getElementById(pageName).style.display = "block";
}

function openEditorPage(pageName, elem) {
    // Hide all elements with class="tabcontent" by default */
    var i, tabcontent, tablinks;
    tabcontent = document.getElementsByClassName("editortabcontent");
    for (i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = "none";
    }

    // Remove the background color of all tablinks/buttons
    tablinks = document.getElementsByClassName("infoblocktablink");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].style.backgroundColor = "";
    }
    elem.style.backgroundColor = '#2196F3';


    // Show the specific tab content
    document.getElementById(pageName).style.display = "block";
}



function update() {
    updateGUI();
    lsg.refresh(editor, editor2);
}


requirejs(['beautifier'],
    function (beautifier) {
        the.beautifier = beautifier;
    });



var the = {
    use_codemirror: !window.location.href.match(/without-codemirror/),
    beautifier_file: window.location.href.match(/debug/) ? 'beautifier' : './beautifier.min',
    beautifier: null,
    beautify_in_progress: false,
    editor: null // codemirror editor
};

opts = {};
opts.indent_size = 4;
opts.indent_char = parseInt(opts.indent_size, 10) === 1 ? '\t' : ' ';
opts.max_preserve_newlines = 5;
opts.preserve_newlines = opts.max_preserve_newlines !== "-1";
opts.keep_array_indentation = false;
opts.break_chained_methods = false;
opts.indent_scripts = 'normal';
opts.brace_style = 'collapse,preserve-inline'; //opts.brace_style = 'expand';
opts.space_before_conditional = true;
opts.unescape_strings = false;
opts.jslint_happy = false;
opts.end_with_newline = false;
opts.wrap_line_length = 0;
opts.indent_inner_html = false;
opts.comma_first = false;
opts.e4x = false;
opts.indent_empty_lines = false;

